@@ -41,17 +41,21 @@ setMethod("dots", signature(x="SpatVector"),
 #	cols <- out$cols
 #	if (is.null(cols)) cols = rep("black", n)
 
-	g <- lapply(x@ptr$linesList(), function(i) { names(i)=c("x", "y"); i } )
+#	g <- lapply(x@ptr$linesList(), function(i) { names(i)=c("x", "y"); i } )
 
 #	g <- geom(x, df=TRUE)
 #	g <- split(g, g[,1])
 #	g <- lapply(g, function(x) split(x[,3:4], x[,2]))
 #	n <- length(g)
 
+	g <- x@ptr$linesList()
 	lty <- rep_len(lty, n)
 	lwd <- rep_len(lwd, n)
 	for (i in 1:n) {
-		graphics::plot.xy(g[[i]], type="l", lty=lty[i], col=out$main_cols[i], lwd=lwd[i], ...)
+		if (!is.null(g[[i]])) {
+			names(g[[i]]) = c("x", "y")
+			graphics::plot.xy(g[[i]], type="l", lty=lty[i], col=out$main_cols[i], lwd=lwd[i], ...)
+		}
 	}
 
 #	for (i in 1:n) {
@@ -115,14 +119,14 @@ setMethod("dots", signature(x="SpatVector"),
 
 	g <- x@ptr$polygonsList()
 	if (is.null(out$leg$density)) {
-		for (i in 1:length(g)) {
-			for (j in 1:length(g[[i]])) {
+		for (i in seq_along(g)) {
+			for (j in seq_along(g[[i]])) {
 				graphics::polypath(g[[i]][[j]][[1]], g[[i]][[j]][[2]], col=out$main_cols[i], rule = "evenodd", border=out$leg$border[i], lwd=out$leg$lwd[i], lty=out$leg$lty[i], ...)
 			}
 		}
 	} else {
 		for (i in 1:length(g)) {
-			for (j in 1:length(g[[i]])) {
+			for (j in seq_along(g[[i]])) {
 				graphics::polygon(g[[i]][[j]][[1]], g[[i]][[j]][[2]], col=out$main_cols[i], density=out$leg$density[i], angle=out$leg$angle[i], border=NA, lwd=out$leg$lwd[i], lty=out$leg$lty[i], ...)
 				graphics::polypath(g[[i]][[j]][[1]], g[[i]][[j]][[2]], col=NA, rule="evenodd", border=out$leg$border[i], lwd=out$leg$lwd[i], lty=out$leg$lty[i], ...)
 			}
@@ -583,14 +587,14 @@ setMethod("plot", signature(x="SpatVector", y="character"),
 				lastrow <- i > (prod(nrnc) - nrnc[2])
 				if (lastrow) {
 					if (newrow) {
-						pax$sides <- 1:2
+						pax$side <- 1:2
 					} else {
-						pax$sides <- 1
+						pax$side <- 1
 					}
 				} else if (newrow) {
-					pax$sides <- 2
+					pax$side <- 2
 				} else {
-					pax$sides <- 0
+					pax$side <- 0
 				}
 			}
 			if (missing(col)) col <- NULL
